/*
  access to physical address
  by izumi@ieee.org ver.2016/10/07a
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/mman.h>
#include <fcntl.h>

int main(int argc,char *argv[]){
  int fd;
  int dirout=0;
  unsigned int value=0;
  unsigned int addr=0;
  unsigned page_addr,page_offset;
  unsigned page_size=sysconf(_SC_PAGESIZE);
  void *ptr;

  if (argc==3) {
    /* output */
    dirout=1;
    addr=strtoul(argv[1],NULL,0);
    value=strtoul(argv[2],NULL,0);
  } else if (argc==2) {
    /* input */
    dirout=0;
    addr=strtoul(argv[1],NULL,0);
  } else {
    printf("IO access through /dev/mem.\n");
    printf("%s <ADDR> [<VALUE>]\n",argv[0]);
    printf("	<ADDR>  IO physical address\n");
    printf("	<VALUE> output value to IO\n");
    return -1;
  }
  
  /* Open /dev/mem file */
  fd=open("/dev/mem",O_RDWR);
  if (fd<=0) {
    fprintf(stderr,"can't open /dev/mem\n");
    return -1;
  }

  /* mmap the memory device */
  page_addr=addr&~(page_size-1);
  page_offset=addr-page_addr;
  ptr=mmap(NULL,page_size,PROT_READ|PROT_WRITE,MAP_SHARED,fd,page_addr);

  if (ptr==0) {
    fprintf(stderr,"failed to mmap the memory device.\n");
    return -1;
  }

  if (dirout) {
    /* write to IO */
    *((unsigned int *)(ptr+page_offset))=value;
    printf("mem[%08x] <= %08x\n",addr,value);
  } else {
    /* read from IO */
    value= *((unsigned int *)(ptr+page_offset));
    printf("mem[%08x] => %08x\n",addr,value);
  }

  munmap(ptr,page_size);
  close(fd);

  return 0;
}
