/*
  access to User Proc device
  by izumi@ieee.org ver.2016/10/07a
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/mman.h>
#include <fcntl.h>

#define SERVO_V_OFS     0x00
#define SERVO_H_OFS     0x04
#define SERVO_DEFAULT   150000
#define SERVO_MIN       120000
#define SERVO_MAX       180000

#define MOTOR_DEFAULT   0
#define MOTOR_MIN       0
#define MOTOR_MAX       65535

#define MOTOR_CTRL_OFS  0x00
#define MOTOR_L_OFS     0x04
#define MOTOR_R_OFS     0x10
#define MOTOR_SA_OFS    0x1C //???
#define MOTOR_SB_OFS    0x20 //???

#define MOTOR_FWDL      0x0001
#define MOTOR_BCKL      0x0003
#define MOTOR_FWDR      0x0300
#define MOTOR_BCKR      0x0100
#define MOTOR_SYNC      0x0080 //???

#define PROC_SERVO "/proc/servo_ctrl"
#define PROC_MOTOR "/proc/motor_ctrl"

int write_proc(int fd, unsigned int ofs, unsigned int val){
  unsigned int buf[2];
  //printf("ofs=%04x,val=%08x\n",ofs,val);
  buf[0]=ofs;
  buf[1]=val;
  return write(fd,(void *)buf,sizeof(unsigned int)*2);
}

int main(int argc,char *argv[]){
  int servo_fd, motor_fd;
 
  servo_fd=open(PROC_SERVO,O_WRONLY);
  if (servo_fd<=0) {
    fprintf(stderr,"can't open servo device file.\n");
    return -1;
  }

  motor_fd=open(PROC_MOTOR,O_WRONLY);
  if (motor_fd<=0) {
    fprintf(stderr,"can't open motor device file.\n");
    return -1;
  }

  // test for servo v
  write_proc(servo_fd,SERVO_V_OFS,SERVO_MIN);      sleep(1);
  write_proc(servo_fd,SERVO_V_OFS,SERVO_MAX);      sleep(1);
  write_proc(servo_fd,SERVO_V_OFS,SERVO_DEFAULT);  sleep(1);

  // test for servo h
  write_proc(servo_fd,SERVO_H_OFS,SERVO_MIN);      sleep(1);
  write_proc(servo_fd,SERVO_H_OFS,SERVO_MAX);      sleep(1);
  write_proc(servo_fd,SERVO_H_OFS,SERVO_DEFAULT);  sleep(1);

  // test for motor h
  write_proc(motor_fd,MOTOR_L_OFS,0.25*MOTOR_MAX);
  write_proc(motor_fd,MOTOR_R_OFS,0.25*MOTOR_MAX);
  write_proc(motor_fd,MOTOR_CTRL_OFS,MOTOR_FWDL);  sleep(1);
  write_proc(motor_fd,MOTOR_CTRL_OFS,MOTOR_FWDR);  sleep(1);
  write_proc(motor_fd,MOTOR_CTRL_OFS,MOTOR_BCKL);  sleep(1);
  write_proc(motor_fd,MOTOR_CTRL_OFS,MOTOR_BCKR);  sleep(1);

  write_proc(motor_fd,MOTOR_L_OFS,0.00*MOTOR_MAX);
  write_proc(motor_fd,MOTOR_R_OFS,0.00*MOTOR_MAX);
  write_proc(motor_fd,MOTOR_CTRL_OFS,MOTOR_FWDL|MOTOR_FWDR); sleep(1);
  write_proc(motor_fd,MOTOR_L_OFS,0.25*MOTOR_MAX);
  write_proc(motor_fd,MOTOR_R_OFS,0.25*MOTOR_MAX);
  write_proc(motor_fd,MOTOR_CTRL_OFS,MOTOR_FWDL|MOTOR_FWDR); sleep(1);
  write_proc(motor_fd,MOTOR_L_OFS,0.50*MOTOR_MAX);
  write_proc(motor_fd,MOTOR_R_OFS,0.50*MOTOR_MAX);
  write_proc(motor_fd,MOTOR_CTRL_OFS,MOTOR_FWDL|MOTOR_FWDR); sleep(1);
  write_proc(motor_fd,MOTOR_L_OFS,0.75*MOTOR_MAX);
  write_proc(motor_fd,MOTOR_R_OFS,0.75*MOTOR_MAX);
  write_proc(motor_fd,MOTOR_CTRL_OFS,MOTOR_FWDL|MOTOR_FWDR); sleep(1);
  write_proc(motor_fd,MOTOR_L_OFS,1.00*MOTOR_MAX);
  write_proc(motor_fd,MOTOR_R_OFS,1.00*MOTOR_MAX);
  write_proc(motor_fd,MOTOR_CTRL_OFS,MOTOR_FWDL|MOTOR_FWDR); sleep(1);

  write_proc(servo_fd,SERVO_V_OFS,SERVO_DEFAULT);
  write_proc(servo_fd,SERVO_H_OFS,SERVO_DEFAULT);
  write_proc(motor_fd,MOTOR_L_OFS,0);
  write_proc(motor_fd,MOTOR_R_OFS,0);
  write_proc(motor_fd,MOTOR_CTRL_OFS,0);

  close(servo_fd);
  close(motor_fd);

  return 0;
}
